@@ -1,10 +1,12 @@
+-- | A collection of predefined combinators. Use these to combine other parsers
+-- into more complex ones.
 module Text.Parcom.Combinators
 ( choice, namedChoice
 , before, between
 , many, many1, manySepBy
 , times
 , skip
-, possibly
+, possibly, optional, option
 )
 where
 
@@ -17,7 +19,7 @@ import Data.Maybe (fromMaybe)
 choice :: (Monad m, Stream s t) => [ParcomT s t m a] -> ParcomT s t m a
 choice xs = foldl (<|>) empty xs <?> "I tried to make a choice, but couldn't"
 
--- | Like @choice@, but each choice tagged with a human-readable name for
+-- | Like 'choice', but each choice tagged with a human-readable name for
 -- better error reporting.
 namedChoice :: (Monad m, Stream s t) => [(String, ParcomT s t m a)] -> ParcomT s t m a
 namedChoice xs = choice (map snd xs) <?> (formatOptionList . map fst) xs
