@@ -1,4 +1,4 @@
-{-#LANGUAGE MultiParamTypeClasses #-}
+{-#LANGUAGE MultiParamTypeClasses, OverloadedStrings #-}
 module Text.Parcom.Text.Strict
 where
 
@@ -14,3 +14,7 @@ instance Stream Text Char where
 instance Listish Text Char where
     toList = unpack
     fromList = pack
+
+instance Textish Text where
+    peekChar "" = (Nothing, 0)
+    peekChar s = (Just (head s), 1)
